var Promise = require('promise');
var configjs = require('./config');
module.exports = function (RED) {

    function ArnidosmarthomeDimmerNode(config) {
        RED.nodes.createNode(this, config);

        console.debug("ArnidosmarthomeDimmerNode")
        console.debug(config);

        this.conf = configjs.getNode();
        this.name = config.name;
        this.device = config.device;
        this.channel = config.channel;
        this.setvalue = parseFloat(config.setvalue);
        this.reverse = config.reverse;
        this.min = config.min;
        this.max = config.max;
        this.mission = config.mission;
        this.operator = config.operator;

        this.currentLevel = 0;

        this.state = {
            value: 0,
            level: 0,
            levelUnit: ""
        }

        this.reportStatus = function (original) {
            let msg = {
                topic: "device",
                payload: parseFloat(this.state.level),
                // {
                //     deviceId: this.channel,
                //     deviceType: "level",
                //     name: this.name,
                //     value: this.state.level,
                //     unit: this.state.levelUnit
                // }
            };

            if (original !== undefined) {
                Object.assign(msg, original);
            }

            this.send(msg);
        }

        // this.setState = function (value) {
        //     Object.assign(this.state, value);

        //     this.reportState();
        // }
        this.updateStatus = function (currentStatus) {
            //if (this.state.level != currentStatus) {
            this.state.level = currentStatus;
            this.reportStatus();
            //}
        }

        //if (this.conf && this.device && this.channel) {
        const callback = (evt) => {
            console.debug("DimmerDevice(" + this.name + ") Callback called");
            console.debug(evt);

            let state = {};

            if (evt["state"] && evt["state"] != "") {
                if (this.mission == "1") { //senaryo başlatıcı ise çıkıştan akış başlatacak
                    this.state.level = evt["state"];
                    if (this.checkCondition()) {
                        this.updateStatus(parseInt(evt["state"]));
                    }                    
                }
                else {
                    this.state.level = (evt["state"]);
                }
            }

        }

        this.conf.registerCallback(this, this.channel, callback);

        this.conf.getDeviceStatus(this.channel).then((status) => {
            console.debug("DimmerDevice(" + this.name + ") Status Refreshed");

            // let state = {};
            this.state.level = status["lastState"];
            // if (status["lastState"] !== undefined) {
            //     state.level = (parseInt(status["lastState"]));
            // }
            // this.updateStatus(state.level)

            //this.setState(state);
        }).catch(err => {
            console.error("Ops... error getting device state (DimmerDevice)");
            console.error(err);
        });

        this.on('input', msg => {
            console.debug("Input Message Received");
            console.log(msg);
            if (this.mission != "") {
                if (this.mission == "0") {//koşul
                    let ret = this.checkCondition();
                    if (ret) {
                        this.reportStatus();
                    }
                }
                if (this.mission == "2") {//cihazı kontrol et
                    if (this.reverse) {
                        let com = this.min != "" && this.max != "" ? (this.state.level == parseInt(this.min) ? this.max : this.min) : this.state.level != 0 ? "0" : "100";
                        this.conf.executeDeviceCommand({
                            channelUid: this.channel,
                            commandType: "Percent",
                            command: com
                        }).then((ret) => {
                            this.updateStatus(parseInt(com));
                        }).catch((ret) => {
                            console.error("Error updating device");
                        });
                    }

                    else if (this.setvalue.toString() != "" && Number.isFinite(this.setvalue)) {
                        this.conf.executeDeviceCommand({
                            channelUid: this.channel,
                            commandType: "Percent",
                            command: (this.setvalue.toString())
                        }).then((ret) => {
                            this.updateStatus(this.setvalue);
                        }).catch((ret) => {
                            console.error("Error updating device");
                        });
                    }
                }
                // else if (msg && msg.payload !== undefined && msg.payload.value != null) {
                //     this.conf.executeDeviceCommand({
                //         channelUid: this.channel,
                //         commandType: "Percent",
                //         command: (msg.payload.value.toString())
                //     }).then((ret) => {
                //         this.updateStatus(parseInt(msg.payload.value));
                //     }).catch((ret) => {
                //         console.error("Error updating device");
                //     });
                // }
            }
        });
        this.checkCondition = function () {
            let ret = false;
            if (this.operator != "") {
                switch (this.operator) {
                    case "0":// =
                        if (parseFloat(this.state.level) == this.setvalue)
                            ret = true;
                        break;
                    case "1":// !=
                        if (parseFloat(this.state.level) != this.setvalue)
                            ret = true;
                        break;
                    case "2":// <
                        if (parseFloat(this.state.level) < this.setvalue)
                            ret = true;
                        break;
                    case "3":// <=
                        if (parseFloat(this.state.level) <= this.setvalue)
                            ret = true;
                        break;
                    case "4":// >
                        if (parseFloat(this.state.level) > this.setvalue)
                            ret = true;
                        break;
                    case "5":// >=
                        if (parseFloat(this.state.level) >= this.setvalue)
                            ret = true;
                        break;

                    default:
                        break;
                }
            }
            else {
                return true;
            }
            return ret;
        }

        this.on('close', () => {
            console.debug("Closed");
            this.conf.unregisterCallback(this, this.channel, callback);
        });
        //}
    }

    RED.nodes.registerType("arnidosmarthome-node-dimmer", ArnidosmarthomeDimmerNode);

};
